## hDQSurv: Transferring Health Models for Survival Analysis
# HealthModels pipeline, version 1.0.0

Orchestrating information across tissues via a novel multi-task GAT framework to improve quantitative gene regulation relation modeling for survival analysis  

Contacts: Fengfeng Zhou; Meiyu Duan  
Email：FengfengZhou@gmail.com, ffzhou@jlu.edu.cn; dmy235813@163.com


## Requirement

anaconda/2020.11  
cuda/10.2  
cudnn/7.6.5.32_cuda10.2  
gcc/7.5  
python/3.7  
torch              1.12.0  
torch-cluster      1.6.0  
torch-geometric    2.1.0.post1   
torch-scatter      2.0.9  
torch-sparse       0.6.15   
torch-spline-conv  1.2.1   
lifelines          0.27.3

## Command

$ python main.py &lt;data folder> &lt;model folder> &lt;output folder> &lt;pretrained>  

*  &lt;data folder>   
Source data folder, each file is csv format.  
data.csv is the transcriptome file, index is the sample IDs, column name is the gene IDs.  
label.csv is the survival label file, index is the sample IDs, columns includes OS and OS.time.

* &lt;model folder>  
HealthModel.pth: The pretrained HealthModel.  
regulatory_geneIDs.csv: The regulatory genes used in this study.  
target_geneIDs.csv: The target genes used in this study.  
adjacent_matrix.csv: The adjacent matrix of regulatory genes.


* &lt;output folder>   
The output files will be written to this folder.


* &lt;pretrained>  
If it is True, DQSurv is initialized using the pre-trained HealthModel weight;   
If it is Flase, DQSurv is initialized randomly.


## Example input execution：
"python main.py ../data ../model ../output True"

The Python environment is 3.7  
The pytorch version is 1.10.0

main.py is the main execution code

## code folder including:

* preprocess.py is the code to process data and label, including deletes the unavaliable data and survival label, splits dataset, KNN imputation and Z-Normalization.

* data_loader.py is the code to load dataset.

* GAT.py is the HealthModel code.
 
* model_mtl.py is the DQSurv code.
 
* PartialNLL.py is the loss function for survival prediction
 
* evaluate.py is the code for training and evaluation of multi-task model
 
* DQSurv.py is the code of pipeline of the framework.
 
* options.py is the code for parameter defination.
 
* survival_utils.py is the code to define early stopper class.

## Output after execution:

* survival_model.pth: The DQSurv model
* survival_param.pth: The DQSurv model parameters
* test_target.csv: The gene expression value of target genes after Z-Normalization and imputation
* pred_target.csv: The predict gene expression value of target genes
* DiseaseM_latent.csv: The latent features constructed by DiseaseModel 
* HealthM_latent.csv: The latent features constructed by HealthModel

These files will be saved in the output folder.  

## Log

slurm-518576.out is the the output of the example data.  
