#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File  : PartialNLL.py
# Author: My Duan
# Date  : 2022-08-16

import torch
import torch.nn as nn
import numpy as np

device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

class PartialNLL(nn.Module):
	def __init__(self):
		super(PartialNLL, self).__init__()

	def forward(self, theta, R, censored):
#        theta = theta.double()
#        R = R.double()
#        censored = censored.double()

		if theta.sum().data == 0.0:
			theta = torch.FloatTensor([1e-8]*theta.size()[0]).unsqueeze(-1)

		exp_theta = torch.exp(theta)
		# observed = (censored.size(0) - torch.sum(censored)).cuda()
		observed = 1 - censored

		# observed = ipcw_weights
		temp = torch.sum((exp_theta * R.t()), 0)
		num_observed = torch.sum(observed)
		loss = -(torch.sum((theta.reshape(-1)- torch.log(temp)) * observed) / num_observed)
#        loss = loss.float()

		if np.isnan(loss.data.tolist()):
			print('PartialNLL')
			for a,b in zip(theta, exp_theta):
				print(a,b)

		return loss
