#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File  : utils.py
# Author: My Duan
# Date  : 2022-09-30

import sys
import pandas as pd
import numpy as np
import torch
import copy

class Logger(object):
	def __init__(self, file_path: str = "./Default.log"):
		self.terminal = sys.stdout
		self.log = open(file_path, "a")

	def write(self, message):
		self.terminal.write(message)
		self.log.write(message)

	def flush(self):
		pass


def transform_na(data: pd.DataFrame, missing_value)-> pd.DataFrame:

	if missing_value == '-9.9658':
		missing_value = float(missing_value)
		thre = 1e-5
		na_data = data.applymap(lambda x: np.nan if x - missing_value < thre else x)
		return na_data

	if missing_value == '0.0':
		missing_value = float(missing_value)
		na_data = data.applymap(lambda x: np.nan if x ==0.0 else x)
		return na_data

	if missing_value == ' ':
		return data

	if missing_value == 'no':
		return data

	else:
		print('We do not support this missing value! Please fill it yourself.')

def _make_R(y):
	# print('hello')
	R = np.zeros((y.shape[0], y.shape[0]))
	for i in range(y.shape[0]):
		for j in range(y.shape[0]):
			R[i,j] = (y[j] >=y[i])
	return R


class EarlyStopper(object):

	def __init__(self, num_trials, model, save_model_path, save_param_path):
		self.num_trials = num_trials
		self.trial_counter = 0
		self.best_loss = 1e5
		self.save_param_path = save_param_path
		self.save_model_path = save_model_path
		self.best_model = model

	def is_continuable(self, model, loss, saved_flag):

		if loss < self.best_loss and self.best_loss - loss > 1e-4:
			self.best_loss = loss
			self.trial_counter = 0
			self.best_model = copy.deepcopy(model)
			if saved_flag == True:
				torch.save(model.state_dict(), self.save_param_path)
				torch.save(model, self.save_model_path)
			return True
		elif self.trial_counter + 1 < self.num_trials:
			self.trial_counter += 1
			return True
		else:
			return False




